@@ -53,27 +53,41 @@ pub struct DayCmd {
     /// 可选。显示详细的使用说明。
     #[argh(option, short = 'h')]
     description: Vec<String>,
+
+    // /// 可选。指定数据库（数据集）的名称，默认为 `rustdx`。
+    // #[argh(option, short = 'd', default = "String::from(\"rustdx\")")]
+    // pub database: String,
+    /// 可选。指定表名称，默认为 `tmp`。
+    #[argh(option, short = 't', default = "String::from(\"tmp\")")]
+    pub table: String,
 }
 
 pub type Stocklist = std::collections::HashSet<String>;
 
 impl DayCmd {
+    // pub const A: &'static str = "";
+
     pub fn run(&self) -> Result<()> {
         match self.output.as_str() {
             "clickhouse" => crate::io::run_clickhouse(self),
-            x if x.ends_with("csv") && self.gbbq.is_some() => {
-                if self.previous.is_some() {
-                    crate::io::run_csv_fq_previous(self)
-                } else {
-                    crate::io::run_csv_fq(self)
-                }
-            }
-            x if x.ends_with("csv") => crate::io::run_csv(self),
+            x if x.ends_with("csv") => self.run_csv(),
             "mongodb" => crate::io::run_mongodb(self),
             _ => todo!(),
         }
     }
 
+    pub fn run_csv(&self) -> Result<()> {
+        if self.gbbq.is_some() {
+            if self.previous.is_some() {
+                crate::io::run_csv_fq_previous(self)
+            } else {
+                crate::io::run_csv_fq(self)
+            }
+        } else {
+            crate::io::run_csv(self)
+        }
+    }
+
     pub fn help_info(&self) -> &Self {
         self.path.iter().map(|p| p.read_dir()).for_each(|p| println!("{:?}", p));
         for arg in &self.description {
