@@ -5,16 +5,13 @@ use rustdx::file::{
     gbbq::{Factor, Gbbq},
 };
 use std::{
-    fs::File,
-    io::Write,
+    fs::{self, File},
+    io::{self, Write},
     path::Path,
     process::Command,
     sync::{Arc, Mutex},
 };
 
-const DATABASE: &'static str = "rustdx";
-const TABLE: &'static str = "rustdx.day_cmd";
-const QUERY: &'static str = "INSERT INTO rustdx.day_cmd FORMAT CSVWithNames";
 const BUFFER_SIZE: usize = 32 * (1 << 20); // 32M
 
 /// TODO 协程解析、异步缓冲写入（利用多核优势）
@@ -47,7 +44,7 @@ pub fn run_csv(cmd: &DayCmd) -> Result<()> {
 /// TODO 协程解析、异步缓冲写入（利用多核优势）
 pub fn run_csv_fq(cmd: &DayCmd) -> Result<()> {
     // 股本变迁
-    let mut bytes = std::fs::read(cmd.gbbq.as_ref().unwrap())?;
+    let mut bytes = fs::read(cmd.gbbq.as_ref().unwrap())?;
     let gbbq = Gbbq::filter_hashmap(Gbbq::iter(&mut bytes[4..]));
 
     // 股票列表
@@ -80,12 +77,15 @@ pub fn run_csv_fq(cmd: &DayCmd) -> Result<()> {
 /// TODO 协程解析、异步缓冲写入（利用多核优势）
 pub fn run_csv_fq_previous(cmd: &DayCmd) -> Result<()> {
     // 股本变迁
-    let mut bytes = std::fs::read(cmd.gbbq.as_ref().unwrap())?;
+    let mut bytes = fs::read(cmd.gbbq.as_ref().unwrap())?;
     let gbbq = Gbbq::filter_hashmap(Gbbq::iter(&mut bytes[4..]));
 
     // 前收
-    let previous =
-        pre_factor(cmd.previous.as_ref().ok_or(anyhow!("请检查“前收盘价”文件的路径"))?)?;
+    let previous = if let Some(Some(path)) = cmd.previous.as_ref().map(|p| p.to_str()) {
+        if path == "clickhouse" { clickhouse_factor(&cmd.table) } else { pre_factor(dbg!(path)) }?
+    } else {
+        return Err(anyhow!("请检查 gbbq 路径"));
+    };
 
     // 股票列表
     let hm = cmd.stocklist();
@@ -151,26 +151,51 @@ pub fn pre_factor(p: impl AsRef<Path>) -> Result<std::collections::HashMap<u32,
                                                .collect())
 }
 
-fn db_setup_clickhouse() -> Result<()> {
-    let create_database = format!("CREATE DATABASE IF NOT EXISTS {}", DATABASE);
+fn database_table(table: &str) -> (&str, &str) {
+    let pos = table.find('.').unwrap();
+    table.split_at(pos) // (database_name, table_name)
+}
+
+fn setup_clickhouse(fq: bool, table: &str) -> Result<()> {
+    let create_database = format!("CREATE DATABASE IF NOT EXISTS {}", database_table(table).0);
     let output = Command::new("clickhouse-client").args(["--query", &create_database]).output()?;
     check_output(output);
     #[rustfmt::skip]
-    let create_table = format!("
-        CREATE TABLE IF NOT EXISTS {}
-        (
-            `date` Date CODEC(DoubleDelta),
-            `code` FixedString(6),
-            `open` Float32,
-            `high` Float32,
-            `low` Float32,
-            `close` Float32,
-            `amount` Float64,
-            `vol` Float64
-        )
-        ENGINE = MergeTree()
-        ORDER BY (date, code)
-    ", TABLE); // PARTITION BY 部分可能需要去掉
+    let create_table = if fq {
+        format!("
+            CREATE TABLE IF NOT EXISTS {}
+            (
+                `date` Date CODEC(DoubleDelta),
+                `code` FixedString(6),
+                `open` Float32,
+                `high` Float32,
+                `low` Float32,
+                `close` Float32,
+                `amount` Float64,
+                `vol` Float64,
+                `preclose` Float64,
+                `factor` Float64
+            )
+            ENGINE = MergeTree()
+            ORDER BY (date, code)
+        ", table)
+    } else {
+        format!("
+            CREATE TABLE IF NOT EXISTS {}
+            (
+                `date` Date CODEC(DoubleDelta),
+                `code` FixedString(6),
+                `open` Float32,
+                `high` Float32,
+                `low` Float32,
+                `close` Float32,
+                `amount` Float64,
+                `vol` Float64
+            )
+            ENGINE = MergeTree()
+            ORDER BY (date, code)
+        ", table)
+    }; // PARTITION BY 部分可能需要去掉
     let output = Command::new("clickhouse-client").args(["--query", &create_table]).output()?;
     check_output(output);
     Ok(())
@@ -179,10 +204,11 @@ fn db_setup_clickhouse() -> Result<()> {
 /// clickhouse-client --query "INSERT INTO table FORMAT CSVWithNames" < clickhouse[.csv]
 pub fn run_clickhouse(cmd: &DayCmd) -> Result<()> {
     use subprocess::{Exec, Redirection};
-    db_setup_clickhouse()?;
-    run_csv(cmd)?;
+    setup_clickhouse(cmd.gbbq.is_some(), &cmd.table)?;
+    cmd.run_csv()?;
     let file = File::open(&cmd.output)?;
-    let capture = Exec::cmd("clickhouse-client").args(&["--query", QUERY])
+    let query = format!("INSERT INTO {} FORMAT CSVWithNames", cmd.table);
+    let capture = Exec::cmd("clickhouse-client").args(&["--query", &query])
                                                 .stdin(Redirection::File(file))
                                                 .capture()?;
     println!("stdout:\n{}stderr:\n{}", capture.stdout_str(), capture.stderr_str());
@@ -191,12 +217,38 @@ pub fn run_clickhouse(cmd: &DayCmd) -> Result<()> {
     Ok(())
 }
 
+/// 获取当前最新 factor
+fn clickhouse_factor(table: &str) -> Result<std::collections::HashMap<u32, Factor>> {
+    let args =
+        ["--query",
+         &format!("WITH (
+                        SELECT max(date) AS date
+                        FROM {0}
+                        ) AS latest
+                    SELECT
+                        date,
+                        code,
+                        close,
+                        factor
+                    FROM {0}
+                    WHERE latest = date
+                    INTO OUTFILE 'factor.csv'
+                    FORMAT CSVWithNames;",
+                  table)];
+    let output = Command::new("clickhouse-client").args(args).output()?;
+    check_output(output);
+    pre_factor("factor.csv")
+}
+
 /// TODO: 与数据库有关的，把库名、表名可配置
 pub fn run_mongodb(cmd: &DayCmd) -> Result<()> {
-    run_csv(cmd)?;
+    cmd.run_csv()?;
     // TODO:排查为什么 date 列无法变成 date 类型 date.date(2006-01-02)
-    let args = ["--db=rustdx",
-                "--collection=day",
+    let (database_name, table_name) = database_table(&cmd.table);
+    let args = ["--db",
+                database_name,
+                "--collection",
+                table_name,
                 "--type=csv",
                 "--file",
                 &cmd.output,
@@ -209,18 +261,17 @@ pub fn run_mongodb(cmd: &DayCmd) -> Result<()> {
 }
 
 fn check_output(output: std::process::Output) {
-    std::io::stdout().write_all(&output.stdout).unwrap();
-    std::io::stderr().write_all(&output.stderr).unwrap();
+    io::stdout().write_all(&output.stdout).unwrap();
+    io::stderr().write_all(&output.stderr).unwrap();
     assert!(output.status.success());
 }
 
-fn keep_csv(fname: &str, keep: bool) -> Result<()> {
+fn keep_csv(fname: &str, keep: bool) -> io::Result<()> {
     if keep {
-        std::fs::rename(fname, format!("{}.csv", fname))?;
+        fs::rename(fname, format!("{}.csv", fname))
     } else {
-        std::fs::remove_file(fname)?;
+        fs::remove_file(fname)
     }
-    Ok(())
 }
 
 /// 读取本地 xls(x) 文件
@@ -316,7 +367,7 @@ pub async fn get_sz_stocks(set: Arc<StockList>) -> Result<usize> {
     let (url, ex) = ("http://www.szse.cn/api/report/ShowReport?\
         SHOWTYPE=xlsx&CATALOGID=1110&TABKEY=tab1&random=0.8587844061443386", "sz");
     let bytes = tokio::spawn(reqwest::get(url).await?.bytes()).await??;
-    let mut workbook = Xlsx::new(std::io::Cursor::new(bytes))?;
+    let mut workbook = Xlsx::new(io::Cursor::new(bytes))?;
     // 每个单元格被解析的类型可能会不一样，所以把股票代码统一转化成字符型
     if let Some(Ok(range)) = workbook.worksheet_range_at(0) {
         set.extend(range.rows().skip(1).map(|r| match &r[4] {
